@@ -9,12 +9,12 @@ HTML::Tiny - Lightweight, dependency free HTML/XML generation
 
 =head1 VERSION
 
-This document describes HTML::Tiny version 0.903
+This document describes HTML::Tiny version 1.00
 
 =cut
 
 use vars qw/$VERSION/;
-$VERSION = '0.903';
+$VERSION = '1.00';
 
 BEGIN {
 
@@ -116,7 +116,7 @@ sub new {
 }
 
 sub _set_auto {
-    my ( $self, $kind, $value ) = @_;
+    my ( $self, $kind, $value ) = splice @_, 0, 3;
     $self->{autotag}->{$kind}->{$_} = $value for @_;
 }
 
@@ -449,8 +449,8 @@ sub stringify {
         }
 
         # ...stringify objects...
-        my $str = eval { $obj->as_string };
-        return $str unless $@;
+        my $str;
+        return $str if eval { $str = $obj->as_string; 1 };
     }
 
     # ...default stringification
@@ -507,15 +507,15 @@ and
 
 =item C<< url_encode( $str ) >>
 
-URL encode a string. Spaces become '+' and unprintable characters are
-encoded as '%' + their hexadecimal character code.
+URL encode a string. Spaces become '+' and non-alphanumeric characters
+are encoded as '%' + their hexadecimal character code.
 
     $h->url_encode( ' <hello> ' )   # returns '+%3chello%3e+'
 =cut
 
 sub url_encode {
     my $str = $_[0]->stringify( $_[1] );
-    $str =~ s/([^A-Za-z0-9_])/$1 eq ' ' ? '+' : sprintf("%%%02x", ord($1))/eg;
+    $str =~ s/([^A-Za-z0-9_~])/$1 eq ' ' ? '+' : sprintf("%%%02x", ord($1))/eg;
     return $str;
 }
 
@@ -569,28 +569,39 @@ would print:
 
 {
     my %ENT_MAP = (
-        '&' => '&amp;',
-        '<' => '&lt;',
-        '>' => '&gt;',
-        '"' => '&quot;',
-        "'" => '&apos;',
+        '&'   => '&amp;',
+        '<'   => '&lt;',
+        '>'   => '&gt;',
+        '"'   => '&#34;', # shorter than &quot;
+        "'"   => '&#39;', # HTML does not define &apos;
+        "\xA" => '&#10;',
+        "\xD" => '&#13;',
     );
 
+    my $text_special = qr/([<>&'"])/;
+    my $attr_special = qr/([<>&'"\x0A\x0D])/; # FIXME needs tests
+
     sub entity_encode {
         my $str = $_[0]->stringify( $_[1] );
-        $str =~ s/([<>&'"])/$ENT_MAP{$1}/eg;
+        my $char_rx = $_[2] ? $attr_special : $text_special;
+        $str =~ s/$char_rx/$ENT_MAP{$1}/eg;
         return $str;
     }
 }
 
-sub _tag_value {
-    my ( $self, $val ) = @_;
-    return '' if ref $val;
-    return '="' . $self->entity_encode( $val ) . '"';
+sub _attr {
+    my ( $self, $attr, $val ) = @_;
+
+    if ( ref $val ) {
+        return $attr if 1; # FIXME correct condition: !$xml_mode
+        $val = $attr;      # FIXME needs tests
+    }
+
+    my $enc_val = $self->entity_encode( $val, 1 );
+    return qq{$attr="$enc_val"};
 }
 
 sub validate_tag {
-
     # Do nothing. Subclass to throw an error for invalid tags
 }
 
@@ -606,11 +617,12 @@ sub _tag {
     $self->validate_tag( $closed, $name, \%attr );
 
     # Generate markup
-    return "<$name"
-      . join( '',
-        map         { ' ' . $_ . $self->_tag_value( $attr{$_} ) }
-          sort grep { defined $attr{$_} } keys %attr )
-      . ( $closed ? ' />' : '>' );
+    my $tag = join( ' ',
+        "<$name",
+        map { $self->_attr( $_, $attr{$_} ) }
+        sort grep { defined $attr{$_} } keys %attr );
+
+    return $tag . ( $closed ? ' />' : '>' ); # FIXME must be $closed && $xml_mode
 }
 
 {
@@ -665,13 +677,14 @@ sub _tag {
 
 Encode a data structure in JSON (Javascript) format:
 
-    print $h->json_encode( { ar => [ 1, 2, 3, { a => 1, b => 2 } ] } )
+    print $h->json_encode( { ar => [ 1, 2, 3, { a => 1, b => 2 } ] } );
 
 would print:
     
     {"ar":[1,2,3,{"a":1,"b":2}]}
 
-Because JSON is valid Javascript this method can be useful when generating ad-hoc Javascript. For example
+Because JSON is valid Javascript this method can be useful when
+generating ad-hoc Javascript. For example
 
     my $some_perl_data = {
         score   => 45,
@@ -736,9 +749,12 @@ L<http://rt.cpan.org>.
 
 Andy Armstrong  C<< <andy@hexten.net> >>
 
+Aristotle Pagaltzis C<< <pagaltzis@gmx.de> >>
+
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2007, Andy Armstrong C<< <andy@hexten.net> >>. All rights reserved.
+Copyright (c) 2007, Andy Armstrong C<< <andy@hexten.net> >>. All
+rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
